@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -17,5 +17,4 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-void rs_ciff_load_meta(const gchar *filename, RS_METADATA *meta);
-GdkPixbuf *rs_ciff_load_thumb(const gchar *src);
+void rs_ciff_load_meta(const gchar *filename, RSMetadata *meta);
