@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -17,6 +17,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#include "rawstudio.h" /* RS_FILETYPE */
+
 #define CONF_LWD "last_working_directory"
 #define CONF_PREBGCOLOR "preview_background_color"
 #define CONF_HISTHEIGHT "histogram_height"
@@ -72,6 +74,7 @@
 #define CONF_TOOLBOX_WIDTH "toolbox_width"
 #define CONF_SPLIT_CONTINUOUS "split_continuous"
 #define CONF_LAST_PRIORITY_PAGE "last_priority_page"
+#define CONF_STORE_SORT_METHOD "store_sort_method"
 
 #define DEFAULT_CONF_EXPORT_DIRECTORY "exports/"
 #define DEFAULT_CONF_EXPORT_FILENAME "%f_%2c"
