@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -47,14 +47,15 @@
 #include "rs-external-editor.h"
 #include "rs-actions.h"
 #include "rs-dir-selector.h"
+#include "rs-metadata.h"
 
 static gchar *filenames[] = {DEFAULT_CONF_EXPORT_FILENAME, "%f", "%f_%c", "%f_output_%4c", NULL};
 static GtkStatusbar *statusbar;
 static gboolean fullscreen;
 GtkWindow *rawstudio_window;
 static gint busycount = 0;
-static GtkWidget *valuefield;
-static GtkWidget *hbox;
+//static GtkWidget *valuefield[3];
+//static GtkWidget *hbox;
 GdkGC *dashed;
 GdkGC *grid;
 
@@ -65,6 +66,7 @@ static gboolean gui_fullscreen_toolbox_callback(GtkWidget *widget, GdkEventWindo
 static void gui_preference_iconview_show_filenames_changed(GtkToggleButton *togglebutton, gpointer user_data);
 static GtkWidget *gui_make_menubar(RS_BLOB *rs);
 static void drag_data_received(GtkWidget *widget, GdkDragContext *drag_context, gint x, gint y, GtkSelectionData *selection_data, guint info, guint t,	RS_BLOB *rs);
+static gboolean gui_window_delete(GtkWidget *widget, GdkEvent  *event, gpointer user_data);
 static GtkWidget *gui_window_make(RS_BLOB *rs);
 static void rs_open_file_delayed(RS_BLOB *rs, const gchar *filename);
 static void rs_open_file(RS_BLOB *rs, const gchar *filename);
@@ -155,29 +157,26 @@ static gboolean
 open_photo(RS_BLOB *rs, const gchar *filename)
 {
 	RS_PHOTO *photo;
-	RS_FILETYPE *filetype;
 	extern GtkLabel *infolabel;
 	gchar *label;
 
-	if ((filetype = rs_filetype_get(filename, TRUE)))
+	rs_preview_widget_set_photo(RS_PREVIEW_WIDGET(rs->preview), NULL);
+	photo = rs_photo_load_from_file(filename, FALSE);
+
+	if (photo)
+		rs_photo_close(rs->photo);
+	else
 	{
-		rs_preview_widget_set_photo(RS_PREVIEW_WIDGET(rs->preview), NULL);
-		photo = rs_photo_load_from_file(filename, FALSE);
-
-		if (photo)
-			rs_photo_close(rs->photo);
-		else
-		{
-			gui_set_busy(FALSE);
-			return FALSE;
-		}
-
-		label = rs_metadata_get_short_description(photo->metadata);
-		gtk_label_set_text(infolabel, label);
-		g_free(label);
-
-		rs_set_photo(rs, photo);
+		gui_set_busy(FALSE);
+		return FALSE;
 	}
+
+	label = rs_metadata_get_short_description(photo->metadata);
+	gtk_label_set_text(infolabel, label);
+	g_free(label);
+
+	rs_set_photo(rs, photo);
+
 	return TRUE;
 }
 
@@ -485,7 +484,7 @@ gui_make_preference_window(RS_BLOB *rs)
 	histsize_hbox = gtk_hbox_new(FALSE, 0);
 	histsize_label = gtk_label_new(_("Histogram height:"));
 	gtk_misc_set_alignment(GTK_MISC(histsize_label), 0.0, 0.5);
-	histsize_adj = gtk_adjustment_new(histogram_height, 15.0, 500.0, 1.0, 10.0, 10.0);
+	histsize_adj = gtk_adjustment_new(histogram_height, 15.0, 500.0, 1.0, 10.0, 0.0);
 	g_signal_connect(histsize_adj, "value_changed",
 		G_CALLBACK(gui_histogram_height_changed), rs);
 	histsize = gtk_spin_button_new(GTK_ADJUSTMENT(histsize_adj), 1, 0);
@@ -757,6 +756,13 @@ drag_data_received(GtkWidget *widget, GdkDragContext *drag_context,
 	return;
 }
 
+static gboolean
+gui_window_delete(GtkWidget *widget, GdkEvent  *event, gpointer user_data)
+{
+	rs_core_action_group_activate("Quit");
+	return TRUE;
+}
+
 static GtkWidget *
 gui_window_make(RS_BLOB *rs)
 {
@@ -765,7 +771,7 @@ gui_window_make(RS_BLOB *rs)
 	rawstudio_window = GTK_WINDOW(gtk_window_new (GTK_WINDOW_TOPLEVEL));
 	gtk_window_resize((GtkWindow *) rawstudio_window, 800, 600);
 	gtk_window_set_title (GTK_WINDOW (rawstudio_window), _("Rawstudio"));
-	g_signal_connect((gpointer) rawstudio_window, "delete_event", G_CALLBACK(rs_shutdown), rs);
+	g_signal_connect((gpointer) rawstudio_window, "delete_event", G_CALLBACK(gui_window_delete), NULL);
 	g_signal_connect((gpointer) rawstudio_window, "key_press_event", G_CALLBACK(window_key_press_event), NULL);
 
 	gtk_drag_dest_set(GTK_WIDGET(rawstudio_window), GTK_DEST_DEFAULT_ALL, targets, 1, GDK_ACTION_COPY);
@@ -787,12 +793,25 @@ preview_wb_picked(RSPreviewWidget *preview, RS_PREVIEW_CALLBACK_DATA *cbdata, RS
 }
 
 void
-preview_motion(RSPreviewWidget *preview, RS_PREVIEW_CALLBACK_DATA *cbdata, RS_BLOB *rs)
+preview_motion(RSPreviewWidget *preview, RS_PREVIEW_CALLBACK_DATA *cbdata, GtkLabel **valuefield)
 {
+	gint c;
 	gchar tmp[20];
 
-	g_snprintf(tmp, 20, "%u %u %u", cbdata->pixel8[R], cbdata->pixel8[G], cbdata->pixel8[B]);
-	gtk_label_set_text(GTK_LABEL(valuefield), tmp);
+	for(c=0;c<3;c++)
+	{
+		g_snprintf(tmp, 20, "%u", cbdata->pixel8[c]);
+		gtk_label_set_markup (valuefield[c], tmp);
+	}
+}
+
+void
+preview_leave(RSPreviewWidget *preview, RS_PREVIEW_CALLBACK_DATA *cbdata, GtkLabel **valuefield)
+{
+	gint c;
+
+	for(c=0;c<3;c++)
+		gtk_label_set_text(valuefield[c], "-");
 }
 
 static gboolean
@@ -903,6 +922,10 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 	GdkColor grid_fg = {0, 32767, 32767, 32767};
 	GdkColor bgcolor = {0, 0, 0, 0 };
 	GString *window_title = g_string_new(_("Rawstudio"));
+	GdkColor tmpcolor;
+	GtkWidget *hbox; /* for statusbar */
+	GtkWidget *valuefield[3];
+
 	
 	gtk_window_set_default_icon_from_file(PACKAGE_DATA_DIR "/icons/" PACKAGE ".png", NULL);
 	rs->window = gui_window_make(rs);
@@ -920,9 +943,30 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 
 	/* Build status bar */
 	statusbar = GTK_STATUSBAR(gtk_statusbar_new());
-	valuefield = gtk_label_new(NULL);
+
+	valuefield[R] = gtk_label_new(NULL);
+	gtk_label_set_width_chars (GTK_LABEL(valuefield[R]), 3);
+	gtk_misc_set_alignment (GTK_MISC(valuefield[R]), 1.0, 0.5);
+	gdk_color_parse("#ef2929", &tmpcolor); /* Scarlet Red */
+	gtk_widget_modify_fg (GTK_WIDGET(valuefield[R]), GTK_STATE_NORMAL, &tmpcolor);
+
+	valuefield[G] = gtk_label_new(NULL);
+	gtk_label_set_width_chars (GTK_LABEL(valuefield[G]), 3);
+	gtk_misc_set_alignment (GTK_MISC(valuefield[G]), 1.0, 0.5);
+	gdk_color_parse("#8ae234", &tmpcolor); /* Chameleon */
+	gtk_widget_modify_fg (GTK_WIDGET(valuefield[G]), GTK_STATE_NORMAL, &tmpcolor);
+
+	valuefield[B] = gtk_label_new(NULL);
+	gtk_label_set_width_chars (GTK_LABEL(valuefield[B]), 3);
+	gtk_misc_set_alignment (GTK_MISC(valuefield[B]), 1.0, 0.5);
+	gdk_color_parse("#729fcf", &tmpcolor); /* Sky Blue */
+	gtk_widget_modify_fg (GTK_WIDGET(valuefield[B]), GTK_STATE_NORMAL, &tmpcolor);
+
 	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), valuefield, FALSE, TRUE, 0);
+	gtk_box_set_spacing(GTK_BOX(hbox), 3);
+	gtk_box_pack_start (GTK_BOX (hbox), valuefield[R], FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), valuefield[G], FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), valuefield[B], FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (statusbar), TRUE, TRUE, 0);
 
 	/* Build toolbox */
@@ -963,7 +1007,8 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 	rs_conf_get_color(CONF_PREBGCOLOR, &bgcolor);
 	rs_preview_widget_set_bgcolor(RS_PREVIEW_WIDGET(rs->preview), &bgcolor);
 	g_signal_connect(G_OBJECT(rs->preview), "wb-picked", G_CALLBACK(preview_wb_picked), rs);
-	g_signal_connect(G_OBJECT(rs->preview), "motion", G_CALLBACK(preview_motion), rs);
+	g_signal_connect(G_OBJECT(rs->preview), "motion", G_CALLBACK(preview_motion), valuefield);
+	g_signal_connect(G_OBJECT(rs->preview), "leave", G_CALLBACK(preview_leave), valuefield);
 
 	/* Split pane below iconbox */
 	pane = gtk_hpaned_new ();
@@ -1040,6 +1085,9 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 		lwd = rs_conf_get_string(CONF_LWD);
 		if (!lwd)
 			lwd = g_get_current_dir();
+
+		/* rs_store_load_directory() MUST have the GDK lock! */
+		gdk_threads_enter();
 		if (rs_store_load_directory(rs->store, lwd))
 		{
 			gint last_priority_page = 0;
@@ -1051,6 +1099,7 @@ gui_init(int argc, char **argv, RS_BLOB *rs)
 		}
 		else
 			rs_conf_set_integer(CONF_LAST_PRIORITY_PAGE, 0);
+		gdk_threads_leave();
 		rs_dir_selector_expand_path(RS_DIR_SELECTOR(dir_selector), lwd);
 		g_free(lwd);
 	}
