@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -71,6 +71,9 @@ rs_fseek(RS_FILE *stream, long offset, int whence)
 		case SEEK_END:
 			stream->offset = stream->size + offset;
 	}
+
+	/* clamp */
+	stream->offset = (stream->offset > stream->size) ? stream->size : ((stream->offset < 0) ? 0 : stream->offset);
 	return(0);
 }
 
