@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -20,7 +20,6 @@
 #ifndef MRW_META_H
 #define MRW_META_H
 
-extern void rs_mrw_load_meta(const gchar *filename, RS_METADATA *meta);
-extern GdkPixbuf *rs_mrw_load_thumb(const gchar *src);
+extern void rs_mrw_load_meta(const gchar *filename, RSMetadata *meta);
 
 #endif
