@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -29,6 +29,9 @@
 #include "rawstudio.h"
 #include "rs-image.h"
 
+#define PITCH(width) ((((width)+15)/16)*16)
+#define SWAP( a, b ) a ^= b ^= a ^= b
+
 struct struct_program {
 	gint divisor;
 	gint scale[9];
@@ -807,6 +810,7 @@ rs_image16_new(const guint width, const guint height, const guint channels, cons
 	rsi->rowstride = rsi->pitch * pixelsize;
 	rsi->channels = channels;
 	rsi->pixelsize = pixelsize;
+	rsi->filters = 0;
 	rsi->pixels = g_new0(gushort, rsi->h*rsi->rowstride);
 	return(rsi);
 }
@@ -920,7 +924,7 @@ convolve_line(RS_IMAGE16 *input, RS_IMAGE16 *output, guint line, struct struct_p
 			+ program->scale[6] * *line2
 			+ program->scale[7] * *line2
 			+ program->scale[8] * *(line2+input->pixelsize);
-		accu /= program->divisor;
+		accu = (accu+128)>>8;
 		_CLAMP65535(accu);
 		*dest = accu;
 		line0++; line1++; line2++; dest++;
@@ -940,7 +944,7 @@ convolve_line(RS_IMAGE16 *input, RS_IMAGE16 *output, guint line, struct struct_p
 			+ program->scale[6] * *(line2-input->pixelsize)
 			+ program->scale[7] * *line2
 			+ program->scale[8] * *(line2+input->pixelsize);
-		accu /= program->divisor;
+		accu = (accu+128)>>8;
 		_CLAMP65535(accu);
 		*dest = accu;
 		line0++; line1++; line2++; dest++;
@@ -959,7 +963,7 @@ convolve_line(RS_IMAGE16 *input, RS_IMAGE16 *output, guint line, struct struct_p
 			+ program->scale[6] * *(line2-input->pixelsize)
 			+ program->scale[7] * *line2
 			+ program->scale[8] * *line2;
-		accu /= program->divisor;
+		accu = (accu+128)>>8;
 		_CLAMP65535(accu);
 		*dest = accu;
 		line0++; line1++; line2++; dest++;
@@ -998,7 +1002,6 @@ rs_image16_convolve(RS_IMAGE16 *input, RS_IMAGE16 *output, RS_MATRIX3 *matrix, g
 	program->scale[6] = (gint) (matrix->coeff[2][0]*256.0);
 	program->scale[7] = (gint) (matrix->coeff[2][1]*256.0);
 	program->scale[8] = (gint) (matrix->coeff[2][2]*256.0);
-	program->divisor = (gint) (scaler * 256.0);
 
 	for(row = 0; row < input->h; row++)
 	{
@@ -1221,7 +1224,7 @@ rs_image16_open_dcraw(const gchar *filename, gboolean half_size)
 	{
 		dcraw_load_raw(raw);
 
-		if (half_size)
+		if (half_size || (raw->filters == 0))
 		{
 			image = rs_image16_new(raw->raw.width, raw->raw.height, raw->raw.colors, 4);
 			rs_image16_open_dcraw_apply_black_and_shift_half_size(raw, image);
@@ -1253,7 +1256,15 @@ rs_image16_open_dcraw_apply_black_and_shift_half_size(dcraw_data *raw, RS_IMAGE1
 {
 	gushort *dst, *src;
 	gint row, col;
-	gint64 shift = (gint64) (16.0-log((gdouble) raw->rgbMax)/log(2.0)+0.5);
+	gint rawsize = raw->raw.width * raw->raw.height * 3;
+	gint max = raw->rgbMax;
+
+	/* dcraw calculates 'wrong' rgbMax for Sigma's, let's calculate our own */
+	if (raw->raw.colors == 3)
+		for(row=0;row<rawsize;row++)
+			max = MAX(((gushort *)raw->raw.image)[row], max);
+
+	gint64 shift = (gint64) (16.0-log((gdouble) max)/log(2.0)+0.5);
 
 	for(row=0;row<(raw->raw.height);row++)
 	{
