@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -31,8 +31,20 @@
 #include "rs-photo.h"
 #include "rs-actions.h"
 #include "rs-job.h"
+#include "rs-metadata.h" /* FIXME: Remove this line and add rs_metadata_get_adobe_matrix() */
+#include "rs-utils.h"
 #include <gettext.h>
 
+enum {
+	ROI_GRID_NONE = 0,
+	ROI_GRID_GOLDEN,
+	ROI_GRID_THIRDS,
+	ROI_GRID_GOLDEN_TRIANGLES1,
+	ROI_GRID_GOLDEN_TRIANGLES2,
+	ROI_GRID_HARMONIOUS_TRIANGLES1,
+	ROI_GRID_HARMONIOUS_TRIANGLES2,
+};
+
 typedef enum {
 	NORMAL           = 0x000F, /* 0000 0000 0000 1111 */
 	WB_PICKER        = 0x0001, /* 0000 0000 0000 0001 */
@@ -139,6 +151,8 @@ struct _RSPreviewWidget
 	GdkPixbuf *buffer[MAX_VIEWS];
 	RSColorTransform *rct[MAX_VIEWS];
 	gint dirty[MAX_VIEWS]; /* Dirty flag, used for multiple things */
+
+	gboolean prev_inside_image; /* For motion and leave function*/
 };
 
 /* Define the boiler plate stuff using the predefined macro */
@@ -156,6 +170,7 @@ G_DEFINE_TYPE (RSPreviewWidget, rs_preview_widget, GTK_TYPE_TABLE);
 enum {
 	WB_PICKED,
 	MOTION_SIGNAL,
+	LEAVE_SIGNAL,
 	LAST_SIGNAL
 };
 
@@ -171,7 +186,8 @@ static void size_allocate (GtkWidget *widget, GtkAllocation *allocation, gpointe
 static void adjustment_changed(GtkAdjustment *adjustment, gpointer user_data);
 static gboolean button(GtkWidget *widget, GdkEventButton *event, RSPreviewWidget *preview);
 static gboolean motion(GtkWidget *widget, GdkEventMotion *event, gpointer user_data);
-static void settings_changed(RS_PHOTO *photo, gint mask, RSPreviewWidget *preview);
+static gboolean leave(GtkWidget *widget, GdkEventCrossing *event, gpointer user_data);
+static void settings_changed(RS_PHOTO *photo, RSSettingsMask mask, RSPreviewWidget *preview);
 static void spatial_changed(RS_PHOTO *photo, RSPreviewWidget *preview);
 static void input_changed(RS_IMAGE16 *image, RSPreviewWidget *preview);
 static void sharpened_changed(RS_IMAGE16 *image, RSPreviewWidget *preview);
@@ -212,6 +228,14 @@ rs_preview_widget_class_init(RSPreviewWidgetClass *klass)
 		NULL,
 		g_cclosure_marshal_VOID__POINTER,
 		G_TYPE_NONE, 1, G_TYPE_POINTER);
+	signals[LEAVE_SIGNAL] = g_signal_new ("leave",
+		G_TYPE_FROM_CLASS (klass),
+		G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+		0, /* Is this right? */
+		NULL,
+		NULL,
+		g_cclosure_marshal_VOID__POINTER,
+		G_TYPE_NONE, 1, G_TYPE_POINTER);
 }
 
 /**
@@ -244,12 +268,15 @@ rs_preview_widget_init(RSPreviewWidget *preview)
 		G_CALLBACK (button), preview);
 	g_signal_connect (G_OBJECT (preview->canvas), "motion_notify_event",
 		G_CALLBACK (motion), preview);
+	g_signal_connect (G_OBJECT (preview->canvas), "leave_notify_event",
+		G_CALLBACK (leave), preview);
 
 	/* Let us know about pointer movements */
 	gtk_widget_set_events(GTK_WIDGET(preview->canvas), 0
 		| GDK_BUTTON_PRESS_MASK
 		| GDK_BUTTON_RELEASE_MASK
-		| GDK_POINTER_MOTION_MASK);
+		| GDK_POINTER_MOTION_MASK
+		| GDK_LEAVE_NOTIFY_MASK);
 
 	preview->state = WB_PICKER;
 	preview->split = SPLIT_VERTICAL;
@@ -295,6 +322,8 @@ rs_preview_widget_init(RSPreviewWidget *preview)
 	g_signal_connect(G_OBJECT(preview->canvas), "size-allocate", G_CALLBACK(size_allocate), preview);
 	g_signal_connect(G_OBJECT(preview), "realize", G_CALLBACK(realize), NULL);
 	g_signal_connect(G_OBJECT(preview->canvas), "scroll_event", G_CALLBACK (scroll), preview);
+
+	preview->prev_inside_image = FALSE;
 }
 
 /**
@@ -883,13 +912,13 @@ get_canvas_placement(RSPreviewWidget *preview, const guint view, GdkRectangle *p
 	if (preview->split == SPLIT_VERTICAL)
 	{
 		xoffset = view * (GTK_WIDGET(preview)->allocation.width/preview->views + SPLITTER_WIDTH/2);
-		width = (width - preview->views*SPLITTER_WIDTH)/preview->views;
+		width = (GTK_WIDGET(preview)->allocation.width - preview->views*SPLITTER_WIDTH)/preview->views;
 	}
 
 	if (preview->split == SPLIT_HORIZONTAL)
 	{
 		yoffset = view * (GTK_WIDGET(preview)->allocation.height/preview->views + SPLITTER_WIDTH/2);
-		height = (height - preview->views*SPLITTER_WIDTH)/preview->views;
+		height = (GTK_WIDGET(preview)->allocation.height - preview->views*SPLITTER_WIDTH)/preview->views;
 	}
 
 	placement->x = xoffset;
@@ -904,8 +933,10 @@ get_placement(RSPreviewWidget *preview, const guint view, GdkRectangle *placemen
 	gint xoffset = 0, yoffset = 0;
 	gint width, height;
 
-	g_return_val_if_fail(preview->scaled[view], FALSE);
-	g_return_val_if_fail(VIEW_IS_VALID(view), FALSE);
+	if (!preview->scaled[view])
+		return FALSE;
+	if (!VIEW_IS_VALID(view))
+		return FALSE;
 
 	width = GTK_WIDGET(preview)->allocation.width;
 	height = GTK_WIDGET(preview)->allocation.height;
@@ -1006,9 +1037,11 @@ buffer(RSPreviewWidget *preview, const gint view, GdkRectangle *dirty)
 	RS_IMAGE16 *source;
 	GdkRectangle clip, image;
 
-	g_return_if_fail(preview->photo);
 	g_return_if_fail(VIEW_IS_VALID(view));
 
+	if (!preview->photo)
+		return;
+
 	if (!ISDIRTY(preview->dirty[view], BUFFER))
 		return;
 
@@ -1853,11 +1886,39 @@ motion(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
 		if (make_cbdata(preview, view, &cbdata, scaled_x, scaled_y, real_x, real_y))
 			g_signal_emit (G_OBJECT (preview), signals[MOTION_SIGNAL], 0, &cbdata);
 	}
+
+	/* Check not to generate superfluous signals "leave"*/
+	if (inside_image != preview->prev_inside_image)
+	{
+		preview->prev_inside_image = inside_image;
+		if (!inside_image && g_signal_has_handler_pending(preview, signals[LEAVE_SIGNAL], 0, FALSE))	
+		{
+			RS_PREVIEW_CALLBACK_DATA cbdata;
+			if (make_cbdata(preview, view, &cbdata, scaled_x, scaled_y, real_x, real_y))
+				g_signal_emit (G_OBJECT (preview), signals[LEAVE_SIGNAL], 0, &cbdata);
+		}
+	}
+
+	return TRUE;
+}
+
+static gboolean 
+leave(GtkWidget *widget, GdkEventCrossing *event, gpointer user_data)
+{
+	RSPreviewWidget *preview = RS_PREVIEW_WIDGET(user_data);
+
+	/* Check not to generate superfluous signals "leave"*/
+	if (preview->prev_inside_image)
+	{
+		preview->prev_inside_image = FALSE;
+		if (g_signal_has_handler_pending(preview, signals[LEAVE_SIGNAL], 0, FALSE))
+			g_signal_emit (G_OBJECT (preview), signals[LEAVE_SIGNAL], 0, NULL);
+	}
 	return TRUE;
 }
 
 static void
-settings_changed(RS_PHOTO *photo, gint mask, RSPreviewWidget *preview)
+settings_changed(RS_PHOTO *photo, RSSettingsMask mask, RSPreviewWidget *preview)
 {
 	gboolean update = FALSE;
 	gint view;
