@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -34,6 +34,9 @@
 #include "eog-pixbuf-cell-renderer.h"
 #include "rs-preload.h"
 #include "rs-photo.h"
+#include "rs-metadata.h"
+#include "rs-filetypes.h"
+#include "rs-utils.h"
 
 /* How many different icon views do we have (tabs) */
 #define NUM_VIEWS 6
@@ -54,6 +57,7 @@ enum {
 	FULLNAME_COLUMN, /* Full path to image */
 	PRIORITY_COLUMN,
 	EXPORTED_COLUMN,
+	METADATA_COLUMN, /* RSMetadata for image */
 	TYPE_COLUMN,
 	GROUP_LIST_COLUMN,
 	NUM_COLUMNS
@@ -76,6 +80,9 @@ struct _RSStore
 	gulong counthandler;
 	gchar *last_path;
 	gboolean cancelled;
+	RS_STORE_SORT_METHOD sort_method;
+	GString *tooltip_text;
+	GtkTreePath *tooltip_last_path;
 };
 
 /* Define the boiler plate stuff using the predefined macro */
@@ -89,6 +96,18 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+typedef struct _worker_job {
+	RSStore *store;
+	gchar *filename;
+	gint priority;
+	gboolean exported;
+	GtkTreeModel *model;
+	GtkTreePath *path;
+} WORKER_JOB;
+
+static GAsyncQueue *loader_queue = NULL;
+static gpointer worker_thread(gpointer data);
+
 /* FIXME: Remember to remove stores from this too! */
 static GList *all_stores = NULL;
 
@@ -104,15 +123,18 @@ static void switch_page(GtkNotebook *notebook, GtkNotebookPage *page, guint page
 static void selection_changed(GtkIconView *iconview, gpointer data);
 static GtkWidget *make_iconview(GtkWidget *iconview, RSStore *store, gint prio);
 static gboolean model_filter_prio(GtkTreeModel *model, GtkTreeIter *iter, gpointer data);
+static gint model_sort_name(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata);
+static gint model_sort_timestamp(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata);
+static gint model_sort_iso(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata);
+static gint model_sort_aperture(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata);
+static gint model_sort_focallength(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata);
+static gint model_sort_shutterspeed(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata);
 static void count_priorities_del(GtkTreeModel *treemodel, GtkTreePath *path, gpointer data);
 static void count_priorities(GtkTreeModel *treemodel, GtkTreePath *do_not_use1, GtkTreeIter *do_not_use2, gpointer data);
 static void icon_get_selected_iters(GtkIconView *iconview, GtkTreePath *path, gpointer user_data);
 static void icon_get_selected_names(GtkIconView *iconview, GtkTreePath *path, gpointer user_data);
 static gboolean tree_foreach_names(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
 static gboolean tree_find_filename(GtkTreeModel *store, const gchar *filename, GtkTreeIter *iter, GtkTreePath **path);
-static void cancel_clicked(GtkButton *button, gpointer user_data);
-GList *find_loadable(const gchar *path, gboolean load_8bit, gboolean load_recursive);
-void load_loadable(RSStore *store, GList *loadable, RS_PROGRESS *rsp);
 void cairo_draw_thumbnail(cairo_t *cr, GdkPixbuf *pixbuf, gint x, gint y, gint width, gint height, gdouble alphas);
 GdkPixbuf * store_group_update_pixbufs(GdkPixbuf *pixbuf, GdkPixbuf *pixbuf_clean);
 void store_group_select_n(GtkListStore *store, GtkTreeIter iter, guint n);
@@ -168,6 +190,16 @@ rs_store_class_init(RSStoreClass *klass)
 		icon_priority_D = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_deleted.png", NULL);
 		icon_exported = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/pixmaps/" PACKAGE "/overlay_exported.png", NULL);
 	}
+
+	if (!loader_queue)
+	{
+		gint i = (rs_get_number_of_processor_cores()*3)/2;
+		loader_queue = g_async_queue_new();
+		while(i--)
+		{
+			g_thread_create(worker_thread, NULL, FALSE, NULL);
+		}
+	}
 }
 
 /**
@@ -193,6 +225,7 @@ rs_store_init(RSStore *store)
 		G_TYPE_STRING,
 		G_TYPE_INT,
 		G_TYPE_BOOLEAN,
+		G_TYPE_OBJECT,
 		G_TYPE_INT,
 		G_TYPE_POINTER);
 
@@ -303,6 +336,11 @@ rs_store_init(RSStore *store)
 	gtk_box_pack_start(GTK_BOX (hbox), GTK_WIDGET(store->notebook), TRUE, TRUE, 0);
 
 	store->last_path = NULL;
+	gint sort_method = RS_STORE_SORT_BY_NAME;
+	rs_conf_get_integer(CONF_STORE_SORT_METHOD, &sort_method);
+	rs_store_set_sort_method(store, sort_method);
+	store->tooltip_text = g_string_new("...");
+	store->tooltip_last_path = NULL;
 }
 
 static void
@@ -432,6 +470,90 @@ selection_changed(GtkIconView *iconview, gpointer data)
 	predict_preload(data, FALSE);
 }
 
+#if GTK_CHECK_VERSION(2,12,0)
+static gboolean
+query_tooltip(GtkWidget *widget, gint x, gint y, gboolean keyboard_mode, GtkTooltip *tooltip, gpointer user_data)
+{
+	gboolean ret = FALSE;
+	RSStore *store = RS_STORE(user_data);
+	GtkIconView *iconview = GTK_ICON_VIEW(widget);
+	GtkTreeModel *model;
+	GtkTreePath *path;
+	GtkScrolledWindow *scrolled_window;
+	GtkAdjustment *adj;
+	GtkTreeIter iter;
+
+	/* Remember the scrollbar - but we only need the horizontal (for now) */
+	scrolled_window = GTK_SCROLLED_WINDOW(gtk_widget_get_parent(widget));
+	adj = GTK_ADJUSTMENT(gtk_scrolled_window_get_hadjustment(scrolled_window));
+	x += (gint) gtk_adjustment_get_value(adj);
+
+	/* See if there's an icon at current position */
+	path = gtk_icon_view_get_path_at_pos(GTK_ICON_VIEW(widget), x, y);
+
+	if (path)
+	{
+		/* If we differ, render a new tooltip text */
+		if (!store->tooltip_last_path || (gtk_tree_path_compare(path, store->tooltip_last_path)!=0))
+		{
+			if (store->tooltip_last_path)
+				gtk_tree_path_free(store->tooltip_last_path);
+			store->tooltip_last_path = path;
+			model = gtk_icon_view_get_model (iconview);
+			if (path && gtk_tree_model_get_iter(model, &iter, path))
+			{
+				RSMetadata *metadata;
+				gint type;
+				gchar *name;
+
+				gtk_tree_model_get (model, &iter,
+					TYPE_COLUMN, &type,
+					TEXT_COLUMN, &name,
+					METADATA_COLUMN, &metadata,
+					-1);
+
+				if (metadata) switch(type)
+				{
+					case RS_STORE_TYPE_GROUP:
+						g_string_printf(store->tooltip_text, "<big>FIXME: group</big>");
+						break;
+					default:
+						g_string_printf(store->tooltip_text, _("<big>%s</big>\n\n"), name);
+
+						if (metadata->focallength > 0)
+							g_string_append_printf(store->tooltip_text, _("<b>Focal length</b>: %dmm\n"), metadata->focallength);
+
+						if (metadata->shutterspeed > 0.0 && metadata->shutterspeed < 4)
+							g_string_append_printf(store->tooltip_text, _("<b>Shutter speed</b>: %.1fs\n"), 1.0/metadata->shutterspeed);
+						else if (metadata->shutterspeed >= 4)
+							g_string_append_printf(store->tooltip_text, _("<b>Shutter speed</b>: 1/%.0fs\n"), metadata->shutterspeed);
+
+						if (metadata->aperture > 0.0)
+							g_string_append_printf(store->tooltip_text, _("<b>Aperture</b>: F/%.01f\n"), metadata->aperture);
+
+						if (metadata->iso != 0)
+							g_string_append_printf(store->tooltip_text, _("<b>ISO</b>: %u\n"), metadata->iso);
+
+						if (metadata->time_ascii != NULL)
+							g_string_append_printf(store->tooltip_text, _("<b>Time</b>: %s"), metadata->time_ascii);
+
+						g_object_unref(metadata);
+						g_free(name);
+						break;
+				}
+			}
+		}
+
+		/* If we're hovering over an icon, we would like to show the tooltip */
+		ret = TRUE;
+	}
+
+	gtk_tooltip_set_markup(tooltip, store->tooltip_text->str);
+
+	return ret;
+}
+#endif /* GTK_CHECK_VERSION(2,12,0) */
+
 static GtkWidget *
 make_iconview(GtkWidget *iconview, RSStore *store, gint prio)
 {
@@ -442,7 +564,8 @@ make_iconview(GtkWidget *iconview, RSStore *store, gint prio)
 
 #if GTK_CHECK_VERSION(2,12,0)
 	/* Enable tooltips */
-	gtk_icon_view_set_tooltip_column(GTK_ICON_VIEW (iconview), TEXT_COLUMN);
+	g_object_set (iconview, "has-tooltip", TRUE, NULL);
+	g_signal_connect(iconview, "query-tooltip", G_CALLBACK(query_tooltip), store);
 #endif
 
 	/* pack them as close af possible */
@@ -519,6 +642,116 @@ model_sort_name(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointe
 	return(ret);
 }
 
+static gint
+model_sort_timestamp(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata)
+{
+	gint ret;
+	RSMetadata *a, *b;
+
+	gtk_tree_model_get(model, tia, METADATA_COLUMN, &a, -1);
+	gtk_tree_model_get(model, tib, METADATA_COLUMN, &b, -1);
+
+	if ((a!=NULL) && (b!=NULL) && (a->timestamp != b->timestamp))
+		ret = a->timestamp - b->timestamp;
+	else
+		ret = model_sort_name(model, tia, tib, userdata);
+
+	if (a!=NULL)
+		g_object_unref(a);
+	if (b!=NULL)
+		g_object_unref(b);
+
+	return(ret);
+}
+
+static gint
+model_sort_iso(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata)
+{
+	gint ret;
+	RSMetadata *a, *b;
+
+	gtk_tree_model_get(model, tia, METADATA_COLUMN, &a, -1);
+	gtk_tree_model_get(model, tib, METADATA_COLUMN, &b, -1);
+
+	if ((a!=NULL) && (b!=NULL) && (a->iso != b->iso))
+		ret = a->iso - b->iso;
+	else
+		ret = model_sort_name(model, tia, tib, userdata);
+
+	if (a!=NULL)
+		g_object_unref(a);
+	if (b!=NULL)
+		g_object_unref(b);
+
+	return(ret);
+}
+
+static gint
+model_sort_aperture(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata)
+{
+	gint ret;
+	RSMetadata *a, *b;
+
+	gtk_tree_model_get(model, tia, METADATA_COLUMN, &a, -1);
+	gtk_tree_model_get(model, tib, METADATA_COLUMN, &b, -1);
+
+	if ((a!=NULL) && (b!=NULL) && (a->aperture != b->aperture))
+		ret = a->aperture*10.0 - b->aperture*10.0;
+	else
+		ret = model_sort_name(model, tia, tib, userdata);
+
+	if (a!=NULL)
+		g_object_unref(a);
+	if (b!=NULL)
+		g_object_unref(b);
+
+	return(ret);
+}
+
+static gint
+model_sort_focallength(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata)
+{
+	gint ret;
+	RSMetadata *a, *b;
+
+	gtk_tree_model_get(model, tia, METADATA_COLUMN, &a, -1);
+	gtk_tree_model_get(model, tib, METADATA_COLUMN, &b, -1);
+
+	if ((a!=NULL) && (b!=NULL) && (a->focallength != b->focallength))
+		ret = a->focallength*10.0 - b->focallength*10.0;
+	else
+		ret = model_sort_name(model, tia, tib, userdata);
+
+	if (a!=NULL)
+		g_object_unref(a);
+	if (b!=NULL)
+		g_object_unref(b);
+
+	return(ret);
+}
+
+static gint
+model_sort_shutterspeed(GtkTreeModel *model, GtkTreeIter *tia, GtkTreeIter *tib, gpointer userdata)
+{
+	gint ret;
+	RSMetadata *a, *b;
+
+	gtk_tree_model_get(model, tia, METADATA_COLUMN, &a, -1);
+	gtk_tree_model_get(model, tib, METADATA_COLUMN, &b, -1);
+
+	if ((a!=NULL) && (b!=NULL) && (a->shutterspeed != b->shutterspeed))
+		ret = b->shutterspeed*10.0 - a->shutterspeed*10.0;
+	else
+		ret = model_sort_name(model, tia, tib, userdata);
+
+	if (a!=NULL)
+		g_object_unref(a);
+	if (b!=NULL)
+		g_object_unref(b);
+
+	return(ret);
+}
+
 static void
 count_priorities_del(GtkTreeModel *treemodel, GtkTreePath *path, gpointer data)
 {
@@ -741,12 +974,77 @@ tree_find_filename(GtkTreeModel *store, const gchar *filename, GtkTreeIter *iter
 	return ret;
 }
 
-
-static void
-cancel_clicked(GtkButton *button, gpointer user_data)
+static gint
+load_directory(RSStore *store, const gchar *path, const gboolean load_8bit, const gboolean load_recursive)
 {
-	RSStore *store = RS_STORE(user_data);
-	store->cancelled = TRUE;
+	const gchar *name;
+	gchar *fullname;
+	GDir *dir;
+	GdkPixbuf *pixbuf;
+	gint count = 0;
+	GtkTreeIter iter;
+	gboolean exported;
+	gint priority;
+
+	pixbuf = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/icons/" PACKAGE ".png", NULL);
+
+	dir = g_dir_open(path, 0, NULL); /* FIXME: check errors */
+
+	while((dir != NULL) && (name = g_dir_read_name(dir)))
+	{
+		/* Ignore "hidden" files and directories */
+		if (name[0] == '.')
+			continue;
+
+		fullname = g_build_filename(path, name, NULL);
+
+		if (rs_filetype_can_load(fullname))
+		{
+			WORKER_JOB *job;
+
+			/* Sane defaults */
+			priority = PRIO_U;
+			exported = FALSE;
+
+			/* Load flags from XML cache */
+			rs_cache_load_quick(fullname, &priority, &exported);
+
+			/* Add thumbnail to store */
+			gtk_list_store_prepend (store->store, &iter);
+			gtk_list_store_set (store->store, &iter,
+				METADATA_COLUMN, NULL,
+				PIXBUF_COLUMN, pixbuf,
+				PIXBUF_CLEAN_COLUMN, pixbuf,
+				TEXT_COLUMN, name,
+				FULLNAME_COLUMN, fullname,
+				PRIORITY_COLUMN, priority,
+				EXPORTED_COLUMN, exported,
+				-1);
+
+			/* Push an asynchronous job for loading the thumbnail */
+			job = g_new(WORKER_JOB, 1);
+			job->store = g_object_ref(store);
+			job->filename = g_strdup(fullname);
+			job->priority = priority;
+			job->exported = exported;
+			job->model = g_object_ref(GTK_TREE_MODEL(store->store));
+			job->path = gtk_tree_model_get_path(GTK_TREE_MODEL(store->store), &iter);
+			g_async_queue_push(loader_queue, job);
+
+			count++;
+		}
+		else if (load_recursive && g_file_test(fullname, G_FILE_TEST_IS_DIR))
+			count += load_directory(store, fullname, load_8bit, load_recursive);
+
+		g_free(fullname);
+	}
+
+	g_object_unref(pixbuf);
+
+	if (dir)
+		g_dir_close(dir);
+
+	return count;
 }
 
 /* Public functions */
@@ -772,6 +1070,11 @@ rs_store_remove(RSStore *store, const gchar *filename, GtkTreeIter *iter)
 {
 	GtkTreeIter i;
 
+	/* Empty the loader queue */
+	g_async_queue_lock(loader_queue);
+	while(g_async_queue_try_pop_unlocked(loader_queue));
+	g_async_queue_unlock(loader_queue);
+
 	/* If we got no store, iterate though all */
 	if (!store)
 	{
@@ -796,143 +1099,7 @@ rs_store_remove(RSStore *store, const gchar *filename, GtkTreeIter *iter)
 	/* If both are NULL, remove everything */
 	if ((filename == NULL) && (iter == NULL))
 		gtk_list_store_clear(store->store);
-}
 
-/**
- * Load thumbnails from a directory into the store
- * @param path The path to load
- * @param load_8bit Boolean
- * @param load_recursive Boolean
- * @return GList containing paths to loadable files
- */
-GList *
-find_loadable(const gchar *path, gboolean load_8bit, gboolean load_recursive)
-{
-	gchar *name;
-	RS_FILETYPE *filetype;
-	GString *fullname;
-	GList *loadable = NULL;
-	GDir *dir = g_dir_open(path, 0, NULL);
-
-	if (dir == NULL)
-		return loadable;
-
-	while ((name = (gchar *) g_dir_read_name(dir)))
-	{
-		fullname = g_string_new(path);
-		fullname = g_string_append(fullname, G_DIR_SEPARATOR_S);
-		fullname = g_string_append(fullname, name);
-		filetype = rs_filetype_get(name, TRUE);
-		if (filetype)
-		{
-			if (filetype->load && ((filetype->filetype==FILETYPE_RAW)||load_8bit))
-			{
-				loadable = g_list_append(loadable, fullname->str);
-			}
-		}
-		if (load_recursive)
-		{
-			if (g_file_test(fullname->str, G_FILE_TEST_IS_DIR))
-			{
-				/* We don't load hidden directories */
-				if (name[0] != '.')
-				{
-					GList *temp_loadable;
-					temp_loadable = find_loadable(fullname->str, load_8bit, load_recursive);
-					loadable = g_list_concat(loadable, temp_loadable);
-				}
-			}
-		}
-		g_string_free(fullname, FALSE);
-	}
-
-	if (dir)
-		g_dir_close(dir);
-
-	return loadable;
-}
-
-/**
- * Load thumbnails from a directory into the store
- * @param store A RSStore
- * @param loadable A GList containing paths to loadable files
- * @param rsp A progress bar
- */
-void
-load_loadable(RSStore *store, GList *loadable, RS_PROGRESS *rsp)
-{
-	gchar *name;
-	RS_FILETYPE *filetype;
-	GdkPixbuf *pixbuf;
-	GdkPixbuf *pixbuf_clean;
-	GdkPixbuf *missing_thumb;
-	gint priority;
-	gboolean exported = FALSE;
-	GtkTreeIter iter;
-	gchar *fullname;
-	gint n;
-
-	/* We will use this, if no thumbnail can be loaded */
-	missing_thumb = gtk_widget_render_icon(GTK_WIDGET(store),
-		GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_DIALOG, NULL);
-
-	for(n = 0; n < g_list_length(loadable); n++)
-	{
-		if (store->cancelled)
-			break;
-		fullname = g_list_nth_data(loadable, n);
-		name = g_path_get_basename(fullname);
-
-		filetype = rs_filetype_get(name, TRUE);
-		if (filetype)
-		{
-			if (filetype->load)
-			{
-				pixbuf = rs_load_thumb(filetype, fullname);
-				if (pixbuf==NULL)
-				{
-					pixbuf = gdk_pixbuf_copy(missing_thumb);
-					g_object_ref (pixbuf);
-				}
-
-				/* Save a clean copy of the thumbnail for later use */
-				pixbuf_clean = gdk_pixbuf_copy(pixbuf);
-
-				/* Sane defaults */
-				priority = PRIO_U;
-				exported = FALSE;
-
-				/* Load flags from XML cache */
-				rs_cache_load_quick(fullname, &priority, &exported);
-
-				/* Update thumbnail */
-				thumbnail_update(pixbuf, pixbuf_clean, priority, exported);
-
-				/* Add thumbnail to store */
-				gtk_list_store_prepend (store->store, &iter);
-				gtk_list_store_set (store->store, &iter,
-					PIXBUF_COLUMN, pixbuf,
-					PIXBUF_CLEAN_COLUMN, pixbuf_clean,
-					TEXT_COLUMN, name,
-					FULLNAME_COLUMN, fullname,
-					PRIORITY_COLUMN, priority,
-					EXPORTED_COLUMN, exported,
-					-1);
-
-				/* We can safely unref pixbuf by now, store holds a reference */
-				g_object_unref (pixbuf);
-				g_object_unref (pixbuf_clean);
-
-				/* Move our progress bar */
-				gui_progress_advance_one(rsp);
-			}
-		}
-		g_free(name);
-	}
-
-	g_object_unref(missing_thumb);
-
-	return;
 }
 
 /**
@@ -948,14 +1115,11 @@ rs_store_load_directory(RSStore *store, const gchar *path)
 	GStaticMutex lock = G_STATIC_MUTEX_INIT;
 
 	GtkTreeSortable *sortable;
-	RS_PROGRESS *rsp;
-	GtkWidget *cancel;
 	gboolean load_8bit = FALSE;
 	gboolean load_recursive = DEFAULT_CONF_LOAD_RECURSIVE;
 	gint items=0, n;
 	GtkTreePath *treepath;
 	GtkTreeIter iter;
-	GList *loadable = NULL;
 
 	g_return_val_if_fail(RS_IS_STORE(store), -1);
 	if (!path)
@@ -982,10 +1146,9 @@ rs_store_load_directory(RSStore *store, const gchar *path)
 	rs_conf_get_boolean(CONF_LOAD_GDK, &load_8bit);
 	rs_conf_get_boolean(CONF_LOAD_RECURSIVE, &load_recursive);
 
-	/* find loadable files */
-	loadable = find_loadable(path, load_8bit, load_recursive);
-	loadable = g_list_first(loadable);
-	items = g_list_length(loadable);
+	/* Block the priority count */
+	g_signal_handler_block(store->store, store->counthandler);
+	items = load_directory(store, path, load_8bit, load_recursive);
 
 	/* unset model and make sure we have enough columns */
 	for (n=0;n<NUM_VIEWS;n++)
@@ -994,21 +1157,6 @@ rs_store_load_directory(RSStore *store, const gchar *path)
 		gtk_icon_view_set_columns(GTK_ICON_VIEW (store->iconview[n]), items);
 	}
 
-	/* Block the priority count */
-	g_signal_handler_block(store->store, store->counthandler);
-
-	/* Add a progress bar */
-	store->cancelled = FALSE;
-	cancel = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-	g_signal_connect (G_OBJECT(cancel), "clicked", G_CALLBACK(cancel_clicked), store);
-	rsp = gui_progress_new_with_delay(_("Opening directory..."), items, 200);
-	gui_progress_add_widget(rsp, cancel);
-
-	/* load all loadable items */
-	load_loadable(store, loadable, rsp);
-	g_list_foreach (loadable, (GFunc)g_free, NULL);
-	g_list_free(loadable);
-
 	/* Sort the store */
 	sortable = GTK_TREE_SORTABLE(store->store);
 	gtk_tree_sortable_set_sort_func(sortable,
@@ -1043,9 +1191,6 @@ rs_store_load_directory(RSStore *store, const gchar *path)
 	store_load_groups(store->store);
 #endif
 
-	/* Free the progress bar */
-	gui_progress_free(rsp);
-
 	/* Start the preloader */
 	predict_preload(store, TRUE);
 
@@ -1446,6 +1591,73 @@ rs_store_get_current_page(RSStore *store)
 	return gtk_notebook_get_current_page(store->notebook);
 }
 
+/**
+ * Sets the sorting method in a RSStore
+ * @param store A RSStore
+ * @param sort A sort method from the RS_STORE_SORT_BY-family of enums
+ */
+void
+rs_store_set_sort_method(RSStore *store, RS_STORE_SORT_METHOD sort_method)
+{
+	GtkTreeSortable *sortable;
+	gint sort_column = TEXT_COLUMN;
+	GtkTreeIterCompareFunc sort_func = model_sort_name;
+
+	g_assert(RS_IS_STORE(store));
+
+	store->sort_method = sort_method;
+	rs_conf_set_integer(CONF_STORE_SORT_METHOD, sort_method);
+
+	switch (sort_method)
+	{
+		case RS_STORE_SORT_BY_NAME:
+			sort_column = TEXT_COLUMN;
+			sort_func = model_sort_name;
+			break;
+		case RS_STORE_SORT_BY_TIMESTAMP:
+			sort_column = METADATA_COLUMN;
+			sort_func = model_sort_timestamp;
+			break;
+		case RS_STORE_SORT_BY_ISO:
+			sort_column = METADATA_COLUMN;
+			sort_func = model_sort_iso;
+			break;
+		case RS_STORE_SORT_BY_APERTURE:
+			sort_column = METADATA_COLUMN;
+			sort_func = model_sort_aperture;
+			break;
+		case RS_STORE_SORT_BY_FOCALLENGTH:
+			sort_column = METADATA_COLUMN;
+			sort_func = model_sort_focallength;
+			break;
+		case RS_STORE_SORT_BY_SHUTTERSPEED:
+			sort_column = METADATA_COLUMN;
+			sort_func = model_sort_shutterspeed;
+			break;
+	}
+
+	sortable = GTK_TREE_SORTABLE(store->store);
+	gtk_tree_sortable_set_sort_func(sortable,
+		sort_column,
+		sort_func,
+		store,
+		NULL);
+	gtk_tree_sortable_set_sort_column_id(sortable, sort_column, GTK_SORT_ASCENDING);
+}
+
+/**
+ * Get the sorting method for a RSStore
+ * @param store A RSStore
+ * @return A sort method from the RS_STORE_SORT_BY-family of enums
+ */
+extern RS_STORE_SORT_METHOD
+rs_store_get_sort_method(RSStore *store)
+{
+	g_assert(RS_IS_STORE(store));
+
+	return store->sort_method;
+}
+
 void
 cairo_draw_thumbnail(cairo_t *cr, GdkPixbuf *pixbuf, gint x, gint y, gint width, gint height, gdouble alpha)
 {
@@ -1937,11 +2149,10 @@ GList
 void
 rs_store_auto_group(RSStore *store)
 {
-	RS_FILETYPE *filetype = NULL;
 	gchar *filename = NULL;
 	gint timestamp = 0, timestamp_old = 0;
 	gint exposure;
-	RS_METADATA *meta;
+	RSMetadata *meta;
 	GList *filenames = NULL;
 	GtkTreeIter iter;
 
@@ -1951,31 +2162,24 @@ rs_store_auto_group(RSStore *store)
 	do
 	{
 		store_get_fullname(GTK_LIST_STORE(store->store), &iter, &filename);
-		filetype = rs_filetype_get(filename, TRUE);
-		if (filetype)
-		{
-			if(filetype->load_meta)
-			{
-				meta = rs_metadata_new();
-				filetype->load_meta(filename, meta);
-				
-				if (!meta->timestamp)
-					return;
-				
-				timestamp = meta->timestamp;
-				exposure = (1/meta->shutterspeed);
+		meta = rs_metadata_new_from_file(filename);
 
-				if (timestamp > timestamp_old + 1)
-				{
-					if (g_list_length(filenames) > 1)
-						store_group_photos_by_filenames(store->store, filenames);
-					g_list_free(filenames);
-					filenames = NULL;
-				}
-				timestamp_old = timestamp + exposure;
-				g_free(meta);
-			}
+		if (!meta->timestamp)
+			return;
+
+		timestamp = meta->timestamp;
+		exposure = (1/meta->shutterspeed);
+
+		if (timestamp > timestamp_old + 1)
+		{
+			if (g_list_length(filenames) > 1)
+				store_group_photos_by_filenames(store->store, filenames);
+			g_list_free(filenames);
+			filenames = NULL;
 		}
+		timestamp_old = timestamp + exposure;
+		g_free(meta);
+
 		filenames = g_list_append(filenames, filename);
 	} while (gtk_tree_model_iter_next(GTK_TREE_MODEL(store->store), &iter));
 
@@ -2121,3 +2325,59 @@ store_set_members(GtkListStore *store, GtkTreeIter *iter, GList *members)
 						GROUP_LIST_COLUMN, members,
 						-1);
 }
+
+static gpointer
+worker_thread(gpointer data)
+{
+	WORKER_JOB *job;
+	GdkPixbuf *pixbuf, *pixbuf_clean;
+	GtkTreeIter iter;
+	RSMetadata *metadata;
+
+	while((job = g_async_queue_pop(loader_queue)))
+	{
+		metadata = rs_metadata_new_from_file(job->filename);
+		g_assert(RS_IS_METADATA(metadata));
+
+		pixbuf = rs_metadata_get_thumbnail(metadata);
+
+		if (pixbuf==NULL)
+			/* We will use this, if no thumbnail can be loaded */
+			pixbuf = gtk_widget_render_icon(GTK_WIDGET(job->store),
+				GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_DIALOG, NULL);
+
+		pixbuf_clean = gdk_pixbuf_copy(pixbuf);
+
+		/* Update thumbnail */
+		thumbnail_update(pixbuf, pixbuf_clean, job->priority, job->exported);
+
+		g_assert(pixbuf != NULL);
+		g_assert(pixbuf_clean != NULL);
+
+		/* Add the new thumbnail to the store */
+		gdk_threads_enter();
+		if (tree_find_filename(job->model, job->filename, &iter, NULL))
+		{
+			gtk_list_store_set(GTK_LIST_STORE(job->model), &iter,
+				METADATA_COLUMN, metadata,
+				PIXBUF_COLUMN, pixbuf,
+				PIXBUF_CLEAN_COLUMN, pixbuf_clean,
+				-1);
+		}
+		gdk_threads_leave();
+
+		/* The GtkListStore should have ref'ed these */
+		g_object_unref(pixbuf);
+		g_object_unref(pixbuf_clean);
+
+		/* Clean up the job */
+		g_free(job->filename);
+		gtk_tree_path_free(job->path);
+		g_object_unref(job->store);
+		g_object_unref(job->model);
+		g_free(job);
+	}
+
+	/* This is only to stop gcc from complaining, we will never return */
+	return NULL;
+}
