@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -26,13 +26,24 @@
 #include "rs-image.h"
 #include "rs-color-transform.h"
 #include "rs-utils.h"
-#include "rs-photo.h"
+#include "rs-filetypes.h"
+#include "rs-metadata.h"
+#include <arpa/inet.h> /* sony_decrypt(): htonl() */
 
 /* It is required having some arbitrary maximum exposure time to prevent borked
  * shutter speed values being interpreted from the tiff.
  * 8h seems to be reasonable, even for astronomists with extra battery packs */
 #define EXPO_TIME_MAXVAL (8*60.0*60.0)
 
+typedef struct {
+	RSMetadata meta;
+	gint sony_offset;
+	gint sony_length;
+	gint sony_key;
+	guint pad[128];
+	guint p;
+} SonyMeta;
+
 struct IFD {
 	gushort tag;
 	gushort type;
@@ -48,16 +59,19 @@ struct IFD {
 
 static gfloat get_rational(RAWFILE *rawfile, guint offset);
 inline static void read_ifd(RAWFILE *rawfile, guint offset, struct IFD *ifd);
-static gboolean makernote_canon(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
-static gboolean makernote_leica(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
-static gboolean makernote_minolta(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
-static gboolean makernote_nikon(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
-static gboolean makernote_olympus(RAWFILE *rawfile, guint base, guint offset, RS_METADATA *meta);
-static gboolean makernote_olympus_camerasettings(RAWFILE *rawfile, guint base, guint offset, RS_METADATA *meta);
-static gboolean makernote_olympus_imageprocessing(RAWFILE *rawfile, guint base, guint offset, RS_METADATA *meta);
-static gboolean makernote_panasonic(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
-static gboolean makernote_pentax(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
-static gboolean ifd_reader(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
+static gboolean makernote_canon(RAWFILE *rawfile, guint offset, RSMetadata *meta);
+static gboolean makernote_leica(RAWFILE *rawfile, guint offset, RSMetadata *meta);
+static gboolean makernote_minolta(RAWFILE *rawfile, guint offset, RSMetadata *meta);
+static gboolean makernote_nikon(RAWFILE *rawfile, guint offset, RSMetadata *meta);
+static gboolean makernote_olympus(RAWFILE *rawfile, guint base, guint offset, RSMetadata *meta);
+static gboolean makernote_olympus_camerasettings(RAWFILE *rawfile, guint base, guint offset, RSMetadata *meta);
+static gboolean makernote_olympus_imageprocessing(RAWFILE *rawfile, guint base, guint offset, RSMetadata *meta);
+static gboolean makernote_panasonic(RAWFILE *rawfile, guint offset, RSMetadata *meta);
+static gboolean makernote_pentax(RAWFILE *rawfile, guint offset, RSMetadata *meta);
+static void sony_decrypt(SonyMeta *sony, guint *data, gint len);
+static gboolean private_sony(RAWFILE *rawfile, guint offset, RSMetadata *meta);
+static gboolean ifd_reader(RAWFILE *rawfile, guint offset, RSMetadata *meta);
+static gboolean thumbnail_reader(const gchar *service, RAWFILE *rawfile, guint offset, guint length, RSMetadata *meta);
 
 typedef enum tiff_field_type
 {
@@ -178,7 +192,7 @@ print_ifd(RAWFILE *rawfile, struct IFD *ifd)
 #endif
 
 static gboolean
-makernote_canon(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
+makernote_canon(RAWFILE *rawfile, guint offset, RSMetadata *meta)
 {
 	gushort number_of_entries = 0;
 	gushort ushort_temp1;
@@ -211,8 +225,12 @@ makernote_canon(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 					case 702: /* Canon EOS 1Ds Mk III */
 					case 796: /* Canon EOS 5D, Canon EOS 30D, Canon EOS 400D */
 					case 1227: /* Canon EOS 450D */
+					case 1250: /* Canon EOS 5D Mk II */
 						ifd.value_offset += 126;
 						break;
+					case 5120: /* Canon PowerShot G10 */
+						ifd.value_offset += 142;
+						break;
 				}
 				/* RGGB-format! */
 				raw_get_ushort(rawfile, ifd.value_offset, &ushort_temp1);
@@ -232,7 +250,7 @@ makernote_canon(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 }
 
 static gboolean
-makernote_leica(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
+makernote_leica(RAWFILE *rawfile, guint offset, RSMetadata *meta)
 {
 	gboolean ret = FALSE;
 	gushort number_of_entries = 0;
@@ -327,7 +345,7 @@ makernote_leica(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 }
 
 static gboolean
-makernote_minolta(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
+makernote_minolta(RAWFILE *rawfile, guint offset, RSMetadata *meta)
 {
 	gushort number_of_entries = 0;
 
@@ -362,7 +380,7 @@ makernote_minolta(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 }
 
 static gboolean
-makernote_nikon(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
+makernote_nikon(RAWFILE *rawfile, guint offset, RSMetadata *meta)
 {
 	static const guchar xlat[2][256] = {
 	{ 0xc1,0xbf,0x6d,0x0d,0x59,0xc5,0x13,0x9d,0x83,0x61,0x6b,0x4f,0xc7,0x7f,0x3d,0x3d,
@@ -455,25 +473,35 @@ makernote_nikon(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 				raw_get_ushort(rawfile, offset+2, &meta->iso);
 				break;
 			case 0x000c: /* D1 White Balance */
-				raw_get_uint(rawfile, offset, &uint_temp1);
+				if (g_str_equal(meta->model_ascii, "NIKON D1X") || g_str_equal(meta->model_ascii, "NIKON D90"))
+				{
+					meta->cam_mul[0] = get_rational(rawfile, offset);
+					meta->cam_mul[2] = get_rational(rawfile, offset+8);
+					meta->cam_mul[1] = get_rational(rawfile, offset+16);
+					meta->cam_mul[3] = get_rational(rawfile, offset+24);
+					rs_metadata_normalize_wb(meta);
+				}
+				else
+				{
+					/* This is fucked, where did these two magic constants come from? */
+					raw_get_float(rawfile, uint_temp1, &float_temp1);
+					raw_get_float(rawfile, uint_temp1+4, &float_temp2);
+					meta->cam_mul[0] = (gdouble) (float_temp1/float_temp2)/1.0;//2.218750;
 
-				/* This is fucked, where did these two magic constants come from? */
-				raw_get_float(rawfile, uint_temp1, &float_temp1);
-				raw_get_float(rawfile, uint_temp1+4, &float_temp2);
-				meta->cam_mul[0] = (gdouble) (float_temp1/float_temp2)/2.218750;
+					raw_get_float(rawfile, uint_temp1+8, &float_temp1);
+					raw_get_float(rawfile, uint_temp1+12, &float_temp2);
+					meta->cam_mul[2] = (gdouble) (float_temp1/float_temp2)/1.0;//1.148438;
 
-				raw_get_float(rawfile, uint_temp1+8, &float_temp1);
-				raw_get_float(rawfile, uint_temp1+12, &float_temp2);
-				meta->cam_mul[2] = (gdouble) (float_temp1/float_temp2)/1.148438;
+					raw_get_float(rawfile, uint_temp1+16, &float_temp1);
+					raw_get_float(rawfile, uint_temp1+20, &float_temp2);
+					meta->cam_mul[1] = (gdouble) (float_temp1/float_temp2);
 
-				raw_get_float(rawfile, uint_temp1+16, &float_temp1);
-				raw_get_float(rawfile, uint_temp1+20, &float_temp2);
-				meta->cam_mul[1] = (gdouble) (float_temp1/float_temp2);
+					raw_get_float(rawfile, uint_temp1+24, &float_temp1);
+					raw_get_float(rawfile, uint_temp1+28, &float_temp2);
+					meta->cam_mul[3] = (gdouble) (float_temp1/float_temp2);
 
-				raw_get_float(rawfile, uint_temp1+24, &float_temp1);
-				raw_get_float(rawfile, uint_temp1+28, &float_temp2);
-				meta->cam_mul[3] = (gdouble) (float_temp1/float_temp2);
-				rs_metadata_normalize_wb(meta);
+					rs_metadata_normalize_wb(meta);
+				}
 				break;
 			case 0x0011: /* NikonPreview */
 				raw_get_uint(rawfile, offset, &uint_temp1);
@@ -481,6 +509,9 @@ makernote_nikon(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 				meta->thumbnail_start += base;
 				break;
 			case 0x0097: /* white balance */
+				if (g_str_equal(meta->model_ascii, "NIKON D90"))
+					break;
+
 				for(i=0;i<4;i++)
 				{
 					raw_get_uchar(rawfile, offset+i, &char_tmp);
@@ -592,7 +623,7 @@ makernote_nikon(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 }
 
 static gboolean
-makernote_olympus_camerasettings(RAWFILE *rawfile, guint base, guint offset, RS_METADATA *meta)
+makernote_olympus_camerasettings(RAWFILE *rawfile, guint base, guint offset, RSMetadata *meta)
 {
 	/* NOTE! At least on E-410 the offsets in this section is relative to
 	   the base of the MakerNotes! */
@@ -642,7 +673,7 @@ makernote_olympus_camerasettings(RAWFILE *rawfile, guint base, guint offset, RS_
 }
 
 static gboolean
-makernote_olympus_imageprocessing(RAWFILE *rawfile, guint base, guint offset, RS_METADATA *meta)
+makernote_olympus_imageprocessing(RAWFILE *rawfile, guint base, guint offset, RSMetadata *meta)
 {
 	gushort number_of_entries;
 	struct IFD ifd;
@@ -684,7 +715,7 @@ makernote_olympus_imageprocessing(RAWFILE *rawfile, guint base, guint offset, RS
 }
 
 static gboolean
-makernote_olympus(RAWFILE *rawfile, guint base, guint offset, RS_METADATA *meta)
+makernote_olympus(RAWFILE *rawfile, guint base, guint offset, RSMetadata *meta)
 {
 	gushort number_of_entries;
 	gushort fieldtag=0;
@@ -719,6 +750,11 @@ makernote_olympus(RAWFILE *rawfile, guint base, guint offset, RS_METADATA *meta)
 		raw_get_uint(rawfile, offset, &uint_temp1);
 		switch(fieldtag)
 		{
+			case 0x0100: /* Thumbnail */
+				raw_get_ushort(rawfile, save-4, &ushort_temp1);
+				meta->thumbnail_start = ushort_temp1;
+				meta->thumbnail_length = valuecount;
+				break;
 			case 0x1017: /* Red multiplier on many Olympus's (E-10, E-300, E-330, E-400, E-500) */
 				raw_get_ushort(rawfile, offset, &ushort_temp1);
 				meta->cam_mul[0] = (gdouble) ushort_temp1 / 256.0;
@@ -742,7 +778,7 @@ makernote_olympus(RAWFILE *rawfile, guint base, guint offset, RS_METADATA *meta)
 }
 
 static gboolean
-makernote_panasonic(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
+makernote_panasonic(RAWFILE *rawfile, guint offset, RSMetadata *meta)
 {
 	gushort number_of_entries;
 	struct IFD ifd;
@@ -789,7 +825,7 @@ makernote_panasonic(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 }
 
 static gboolean
-makernote_pentax(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
+makernote_pentax(RAWFILE *rawfile, guint offset, RSMetadata *meta)
 {
 	gushort number_of_entries;
 	gushort ushort_temp1=0;
@@ -830,8 +866,106 @@ makernote_pentax(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 	return TRUE;
 }
 
+static void
+sony_decrypt(SonyMeta *sony, guint *data, gint len)
+{
+	while (len--)
+		*data++ ^= sony->pad[sony->p++ & 127] = sony->pad[(sony->p+1) & 127] ^ sony->pad[(sony->p+65) & 127];
+}
+
+static gboolean
+private_sony(RAWFILE *rawfile, guint offset, RSMetadata *meta)
+{
+	gushort number_of_entries;
+	struct IFD ifd;
+	gint i;
+	gint key;
+	SonyMeta *sony = (SonyMeta *) meta;
+
+	if(!raw_get_ushort(rawfile, offset, &number_of_entries))
+		return FALSE;
+
+	if (number_of_entries>5000)
+		return FALSE;
+
+	offset += 2;
+
+	while(number_of_entries--)
+	{
+		read_ifd(rawfile, offset, &ifd);
+		offset += 12;
+
+		switch(ifd.tag)
+		{
+			case 0x7200: /* SR2SubIFDOffset */
+				sony->sony_offset = ifd.value_offset;
+				break;
+			case 0x7201: /* SR2SubIFDLength */
+				sony->sony_length = ifd.value_offset;
+				break;
+			case 0x7221: /* SR2SubIFDKey */
+				sony->sony_key = ifd.value_offset;
+
+				/* Initialize decrypter */
+				key = sony->sony_key;
+				for (sony->p=0; sony->p < 4; sony->p++)
+					sony->pad[sony->p] = key = key * 48828125 + 1;
+				sony->pad[3] = sony->pad[3] << 1 | (sony->pad[0]^sony->pad[2]) >> 31;
+				for (sony->p=4; sony->p < 127; sony->p++)
+					sony->pad[sony->p] = (sony->pad[sony->p-4]^sony->pad[sony->p-2]) << 1 | (sony->pad[sony->p-3]^sony->pad[sony->p-1]) >> 31;
+				for (sony->p=0; sony->p < 127; sony->p++)
+					sony->pad[sony->p] = htonl(sony->pad[sony->p]);
+				break;
+		}
+	}
+
+	if ((sony->sony_offset > 0) && (sony->sony_length > 0) && (sony->sony_key != 0))
+	{
+		gpointer buf = g_new0(guchar, sony->sony_length);
+		if (raw_strcpy(rawfile, sony->sony_offset, buf, sony->sony_length))
+		{
+			sony_decrypt(sony, buf, sony->sony_length/4);
+			gushort *sbuf = (gushort *)(buf);
+			gushort tag_count = sbuf[0];
+			struct IFD *private_ifd;
+
+			for(i=0;i<tag_count;i++)
+			{
+#if BYTE_ORDER == BIG_ENDIAN
+#warning FIXME: This will NOT work as expected on a big endian host
+#endif
+				private_ifd = (struct IFD *) (buf+2+i*12);
+
+				switch (private_ifd->tag)
+				{
+					case 0x7303: /* WB_GRBGLevels */
+						sbuf = (gushort *)(buf + private_ifd->value_offset - sony->sony_offset);
+						meta->cam_mul[1] = (gdouble) sbuf[0];
+						meta->cam_mul[0] = (gdouble) sbuf[1];
+						meta->cam_mul[2] = (gdouble) sbuf[2];
+						meta->cam_mul[3] = (gdouble) sbuf[3];
+
+						rs_metadata_normalize_wb(meta);
+						break;
+					case 0x7313: /* WB_RGGBLevels */
+						sbuf = (gushort *)(buf + private_ifd->value_offset - sony->sony_offset);
+						meta->cam_mul[0] = (gdouble) sbuf[0];
+						meta->cam_mul[1] = (gdouble) sbuf[1];
+						meta->cam_mul[3] = (gdouble) sbuf[2];
+						meta->cam_mul[2] = (gdouble) sbuf[3];
+
+						rs_metadata_normalize_wb(meta);
+						break;
+				}
+			}
+		}
+	}
+
+	return TRUE;
+}
+
 gboolean
-exif_reader(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
+exif_reader(RAWFILE *rawfile, guint offset, RSMetadata *meta)
 {
 	gushort number_of_entries = 0;
 
@@ -912,7 +1046,7 @@ exif_reader(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 }
 
 static gboolean
-ifd_reader(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
+ifd_reader(RAWFILE *rawfile, guint offset, RSMetadata *meta)
 {
 	gushort number_of_entries = 0;
 	gboolean is_preview = FALSE;
@@ -979,10 +1113,12 @@ ifd_reader(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 						meta->make = MAKE_PHASEONE;
 					else if (raw_strcmp(rawfile, ifd.value_offset, "SAMSUNG", 7))
 						meta->make = MAKE_SAMSUNG;
-					else if (raw_strcmp(rawfile, ifd.value_offset, "SONY", 4))
-						meta->make = MAKE_SONY;
+					/* Do not detect SONY, we don't want to call private_sony() unless
+					   we're sure we have a hidden SonyMeta */
 					else if (raw_strcmp(rawfile, ifd.value_offset, "FUJIFILM", 4))
 						meta->make = MAKE_FUJIFILM;
+					else if (raw_strcmp(rawfile, ifd.value_offset, "SEIKO EPSON", 11))
+						meta->make = MAKE_EPSON;
 				}
 				break;
 			case 0x0110: /* Model */
@@ -1037,14 +1173,19 @@ ifd_reader(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 
 			/* The following tags are from the DNG spec, they should be safe */
 			case 0xc628: /* DNG: AsShotNeutral */
-				if (ifd.type == TIFF_FIELD_TYPE_RATIONAL && ifd.count == 3)
+				if (((ifd.type == TIFF_FIELD_TYPE_RATIONAL)||(ifd.type == TIFF_FIELD_TYPE_SRATIONAL)) && ifd.count == 3)
 				{
 					meta->cam_mul[0] = 1.0/get_rational(rawfile, ifd.value_offset);
 					meta->cam_mul[1] = 1.0/get_rational(rawfile, ifd.value_offset+8);
 					meta->cam_mul[2] = 1.0/get_rational(rawfile, ifd.value_offset+16);
 					meta->cam_mul[3] = meta->cam_mul[1];
+					rs_metadata_normalize_wb(meta);
 				}
 				break;
+			case 0xc634: /* DNG: PrivateData */
+				if (meta->make == MAKE_SONY)
+					private_sony(rawfile, ifd.value_offset, meta);
+				break;
 		}
 	}
 
@@ -1052,7 +1193,7 @@ ifd_reader(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 }
 
 void
-rs_tiff_load_meta_from_rawfile(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
+rs_tiff_load_meta_from_rawfile(RAWFILE *rawfile, guint offset, RSMetadata *meta)
 {
 	guint next = 0;
 	gushort ifd_num = 0;
@@ -1098,7 +1239,7 @@ rs_tiff_load_meta_from_rawfile(RAWFILE *rawfile, guint offset, RS_METADATA *meta
 }
 
 void
-rs_tiff_load_meta(const gchar *filename, RS_METADATA *meta)
+rs_tiff_load_meta(const gchar *filename, RSMetadata *meta)
 {
 	RAWFILE *rawfile;
 
@@ -1109,46 +1250,31 @@ rs_tiff_load_meta(const gchar *filename, RS_METADATA *meta)
 
 	rs_tiff_load_meta_from_rawfile(rawfile, 0, meta);
 
-	raw_close_file(rawfile);
-}
-
-GdkPixbuf *
-rs_tiff_load_thumb(const gchar *src)
-{
-	RAWFILE *rawfile;
-	GdkPixbuf *pixbuf=NULL, *pixbuf2=NULL;
-	guint start=0, length=0;
-	RS_METADATA *meta = rs_metadata_new();
-
-	if (!(rawfile = raw_open_file(src)))
-		return(NULL);
-
-	rs_tiff_load_meta_from_rawfile(rawfile, 0, meta);
-
-	if ((meta->thumbnail_start>0) && (meta->thumbnail_length>0))
-	{
-		start = meta->thumbnail_start;
-		length = meta->thumbnail_length;
-	}
-
-	else if ((meta->preview_start>0) && (meta->preview_length>0))
-	{
-		start = meta->preview_start;
-		length = meta->preview_length;
-	}
-
 	/* Phase One and Samsung doesn't set this */
 	if ((meta->make == MAKE_PHASEONE) || (meta->make == MAKE_SAMSUNG))
 		meta->preview_planar_config = 1;
 
-	if ((start>0) && (length>0) && (length<5000000))
+	/* Load thumbnail - try thumbnail first - then preview image */
+	if (!thumbnail_reader(filename, rawfile, meta->thumbnail_start, meta->thumbnail_length, meta))
+		thumbnail_reader(filename, rawfile, meta->preview_start, meta->preview_length, meta);
+
+	raw_close_file(rawfile);
+}
+
+static gboolean
+thumbnail_reader(const gchar *service, RAWFILE *rawfile, guint offset, guint length, RSMetadata *meta)
+{
+	gboolean ret = FALSE;
+	GdkPixbuf *pixbuf=NULL, *pixbuf2=NULL;
+
+	if ((offset>0) && (length>0) && (length<5000000))
 	{
 		if ((length==165888) && (meta->make == MAKE_CANON))
-			pixbuf = gdk_pixbuf_new_from_data(raw_get_map(rawfile)+start, GDK_COLORSPACE_RGB, FALSE, 8, 288, 192, 288*3, NULL, NULL);
+			pixbuf = gdk_pixbuf_new_from_data(raw_get_map(rawfile)+offset, GDK_COLORSPACE_RGB, FALSE, 8, 288, 192, 288*3, NULL, NULL);
 		else if (length==57600) /* Multiple Nikon, Pentax and Samsung cameras */
-			pixbuf = gdk_pixbuf_new_from_data(raw_get_map(rawfile)+start, GDK_COLORSPACE_RGB, FALSE, 8, 160, 120, 160*3, NULL, NULL);
+			pixbuf = gdk_pixbuf_new_from_data(raw_get_map(rawfile)+offset, GDK_COLORSPACE_RGB, FALSE, 8, 160, 120, 160*3, NULL, NULL);
 		else if (length==48672)
-			pixbuf = gdk_pixbuf_new_from_data(raw_get_map(rawfile)+start, GDK_COLORSPACE_RGB, FALSE, 8, 156, 104, 156*3, NULL, NULL);
+			pixbuf = gdk_pixbuf_new_from_data(raw_get_map(rawfile)+offset, GDK_COLORSPACE_RGB, FALSE, 8, 156, 104, 156*3, NULL, NULL);
 		else
 			/* Many RAW files are based on TIFF and include the preview image
 			 * as the "main" image in the TIFF so that "normal" image viewing
@@ -1157,7 +1283,7 @@ rs_tiff_load_thumb(const gchar *src)
 			 * possible format (e.g. uncompressed R8G8B8) and use it
 			 * if it is present.
 			 */
-			if (start == meta->preview_start && /* if we're using the preview image */
+			if (offset == meta->preview_start && /* if we're using the preview image */
 				meta->preview_planar_config == 1 && /* uncompressed */
 				meta->preview_bits [0] == 8 &&
 				meta->preview_bits [1] == 8 &&
@@ -1168,26 +1294,26 @@ rs_tiff_load_thumb(const gchar *src)
 				meta->preview_height > 16 &&
 				meta->preview_height < 1024)    /* Some arbitrary sane limit */
 				pixbuf = gdk_pixbuf_new_from_data(
-					raw_get_map(rawfile)+start, GDK_COLORSPACE_RGB, FALSE, 8,
+					raw_get_map(rawfile)+offset, GDK_COLORSPACE_RGB, FALSE, 8,
 					meta->preview_width, meta->preview_height,
 					meta->preview_width * 3, NULL, NULL);
 			else
 				/* Try to guess file format based on contents (JPEG previews) */
-			pixbuf = raw_get_pixbuf(rawfile, start, length);
+			pixbuf = raw_get_pixbuf(rawfile, offset, length);
 	}
 	/* Special case for Panasonic - most have no embedded thumbnail */
 	else if (meta->make == MAKE_PANASONIC)
 	{
-		RS_PHOTO *photo;
-		if ((photo = rs_photo_load_from_file(src, TRUE)))
+		RS_IMAGE16 *input;
+		if ((input = rs_filetype_load(service, TRUE)))
 		{
 			gint c;
 			gfloat pre_mul[4];
 			RS_IMAGE16 *image;
 			RSColorTransform *rct = rs_color_transform_new();
-			image = rs_image16_transform(photo->input, NULL,
-				NULL, NULL, photo->crop, 128, 128, TRUE, -1.0,
-				photo->angle, photo->orientation, NULL);
+			image = rs_image16_transform(input, NULL,
+				NULL, NULL, NULL, 128, 128, TRUE, -1.0,
+				0.0, 0, NULL);
 			pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, image->w, image->h);
 
 			for(c=0;c<4;c++)
@@ -1198,7 +1324,7 @@ rs_tiff_load_thumb(const gchar *src)
 					image->rowstride, gdk_pixbuf_get_pixels(pixbuf),
 					gdk_pixbuf_get_rowstride(pixbuf));
 
-			g_object_unref(photo);
+			g_object_unref(input);
 			g_object_unref(image);
 			g_object_unref(rct);
 		}
@@ -1241,10 +1367,23 @@ rs_tiff_load_thumb(const gchar *src)
 				pixbuf = pixbuf2;
 				break;
 		}
+		meta->thumbnail = pixbuf;
+		ret = TRUE;
 	}
 
-	rs_metadata_free(meta);
-	raw_close_file(rawfile);
+	return ret;
+}
 
-	return(pixbuf);
+void
+rs_sony_load_meta(const gchar *filename, RSMetadata *meta)
+{
+	SonyMeta sony;
+	sony.sony_offset = 0;
+	sony.sony_length = 0;
+	sony.sony_key = 0;
+	meta->make = MAKE_SONY;
+
+	memcpy(&sony, meta, sizeof(RSMetadata));
+	rs_tiff_load_meta(filename, RS_METADATA(&sony));
+	memcpy(meta, &sony, sizeof(RSMetadata));
 }
