@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
 
 #include "rawstudio.h"
 
-extern gboolean exif_reader(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
-extern void rs_tiff_load_meta_from_rawfile(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
-extern void rs_tiff_load_meta(const gchar *filename, RS_METADATA *meta);
-extern GdkPixbuf *rs_tiff_load_thumb(const gchar *src);
+extern gboolean exif_reader(RAWFILE *rawfile, guint offset, RSMetadata *meta);
+extern void rs_tiff_load_meta_from_rawfile(RAWFILE *rawfile, guint offset, RSMetadata *meta);
+extern void rs_tiff_load_meta(const gchar *filename, RSMetadata *meta);
+extern void rs_sony_load_meta(const gchar *filename, RSMetadata *meta);
