@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2008 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006-2009 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -22,6 +22,7 @@
 #include "rawfile.h"
 #include "x3f-meta.h"
 #include "rs-utils.h"
+#include "rs-metadata.h"
 
 /* http://www.x3f.info/technotes/FileDocs/X3F_Format.pdf */
 
@@ -90,6 +91,7 @@ typedef struct x3f_image_data {
 	guint columns;
 	guint rows;
 	guint rowstride; /* row size in bytes, a value of zero means variable */
+	void *image_data;
 	/* Followed by image-data */
 } __attribute__ ((packed)) X3F_IMAGE_DATA;
 
@@ -110,7 +112,7 @@ typedef struct x3f_property {
 } __attribute__ ((packed)) X3F_PROPERTY_ENTRY;
 
 void
-rs_x3f_load_meta(const gchar *filename, RS_METADATA *meta)
+rs_x3f_load_meta(const gchar *filename, RSMetadata *meta)
 {
 	gint i;
 	RAWFILE *rawfile;
@@ -118,6 +120,9 @@ rs_x3f_load_meta(const gchar *filename, RS_METADATA *meta)
 	X3F_DIRECTORY_SECTION directory;
 	X3F_DIRECTORY_ENTRY directory_entry;
 	X3F_IMAGE_DATA image_data;
+	guint start=0, width=0, height=0, rowstride=0;
+	GdkPixbuf *pixbuf = NULL, *pixbuf2 = NULL;
+	gdouble ratio=1.0;
 
 	rawfile = raw_open_file(filename);
 
@@ -136,6 +141,8 @@ rs_x3f_load_meta(const gchar *filename, RS_METADATA *meta)
 	raw_get_uint(rawfile, G_STRUCT_OFFSET(X3F_FILE, rotation), &file.rotation);
 	raw_get_uint(rawfile, raw_get_filesize(rawfile)-4, &file.directory_start);
 
+	meta->orientation=file.rotation;
+
 	if ((file.version_major == 2) && (file.version_minor == 2))
 	{
 		/* Copy all data types in one go */
@@ -184,14 +191,16 @@ rs_x3f_load_meta(const gchar *filename, RS_METADATA *meta)
 				raw_get_uint(rawfile, offset+G_STRUCT_OFFSET(X3F_DIRECTORY_ENTRY, offset), &directory_entry.offset);
 				raw_get_uint(rawfile, offset+G_STRUCT_OFFSET(X3F_DIRECTORY_ENTRY, length), &directory_entry.length);
 
-				if (raw_strcmp(rawfile, offset+G_STRUCT_OFFSET(X3F_DIRECTORY_ENTRY, type), "IMAG", 4))
+				if (raw_strcmp(rawfile, offset+G_STRUCT_OFFSET(X3F_DIRECTORY_ENTRY, type), "IMA", 3))
 				{
 					/* Image Data */
-					raw_get_uint(rawfile, directory_entry.offset+G_STRUCT_OFFSET(X3F_IMAGE_DATA, type_of_image_data), &image_data.type_of_image_data);
-
-					if (image_data.type_of_image_data == X3F_DATA_FORMAT_UNCOMPRESSED)
+					raw_get_uint(rawfile, directory_entry.offset+G_STRUCT_OFFSET(X3F_IMAGE_DATA, data_format), &image_data.data_format);
+					if (image_data.data_format == X3F_DATA_FORMAT_UNCOMPRESSED)
 					{
-						/* FIXME: thumbnail */
+						start = directory_entry.offset+G_STRUCT_OFFSET(X3F_IMAGE_DATA, image_data);
+						raw_get_uint(rawfile, directory_entry.offset+G_STRUCT_OFFSET(X3F_IMAGE_DATA, columns), &width);
+						raw_get_uint(rawfile, directory_entry.offset+G_STRUCT_OFFSET(X3F_IMAGE_DATA, rows), &height);
+						raw_get_uint(rawfile, directory_entry.offset+G_STRUCT_OFFSET(X3F_IMAGE_DATA, rowstride), &rowstride);
 					}
 				}
 				else if (raw_strcmp(rawfile, offset+G_STRUCT_OFFSET(X3F_DIRECTORY_ENTRY, type), "PROP", 4))
@@ -265,50 +274,6 @@ rs_x3f_load_meta(const gchar *filename, RS_METADATA *meta)
 			}
 		}
 	}
-}
-
-GdkPixbuf *
-rs_x3f_load_thumb(const gchar *src)
-{
-	GdkPixbuf *pixbuf = NULL, *pixbuf2 = NULL;
-	gdouble ratio=1.0;
-	guint directory=0, directory_entries=0, n=0;
-	guint data_offset=0, data_length=0, data_format=0;
-	guint start=0, width=0, height=0, rowstride=0;
-	RAWFILE *rawfile;
-
-	raw_init();
-
-	rawfile = raw_open_file(src);
-	if (!rawfile) return(NULL);
-	if (!raw_strcmp(rawfile, 0, "FOVb", 4))
-	{
-		raw_close_file(rawfile);
-		return(NULL);
-	}
-
-	raw_set_byteorder(rawfile, 0x4949); /* x3f is always little endian */
-	raw_get_uint(rawfile, raw_get_filesize(rawfile)-4, &directory);
-	raw_get_uint(rawfile, directory+8, &directory_entries);
-	for(n=0;n<(directory_entries*12);n+=12)
-	{
-		raw_get_uint(rawfile, directory+12+n, &data_offset);
-		raw_get_uint(rawfile, directory+12+n+4, &data_length);
-		if (raw_strcmp(rawfile, directory+12+n+8, "IMA", 3)) /* Catch both IMAG and IMA2 */
-		{
-			if (raw_strcmp(rawfile, data_offset, "SECi", 4))
-			{
-				raw_get_uint(rawfile, data_offset+12, &data_format);
-				if (data_format == 3)
-				{
-					raw_get_uint(rawfile, data_offset+16, &width);
-					raw_get_uint(rawfile, data_offset+20, &height);
-					raw_get_uint(rawfile, data_offset+24, &rowstride);
-					start = data_offset+28;
-				}
-			}
-		}
-	}
 
 	if (width > 0)
 		pixbuf = gdk_pixbuf_new_from_data(raw_get_map(rawfile)+start, GDK_COLORSPACE_RGB, FALSE, 8,
@@ -316,15 +281,21 @@ rs_x3f_load_thumb(const gchar *src)
 
 	if (pixbuf)
 	{
+		if (file.rotation > 0)
+		{
+			pixbuf2 = gdk_pixbuf_rotate_simple(pixbuf,360-file.rotation);
+			g_object_unref(pixbuf);
+			pixbuf = pixbuf2;
+		}
 		ratio = ((gdouble) gdk_pixbuf_get_width(pixbuf))/((gdouble) gdk_pixbuf_get_height(pixbuf));
 		if (ratio>1.0)
 			pixbuf2 = gdk_pixbuf_scale_simple(pixbuf, 128, (gint) (128.0/ratio), GDK_INTERP_BILINEAR);
 		else
 			pixbuf2 = gdk_pixbuf_scale_simple(pixbuf, (gint) (128.0*ratio), 128, GDK_INTERP_BILINEAR);
 		g_object_unref(pixbuf);
-		pixbuf = pixbuf2;
+		meta->thumbnail = pixbuf2;
 	}
 
 	raw_close_file(rawfile);
-	return(pixbuf);
+	return;
 }
